@@ -1,16 +1,6 @@
 # -*- coding: UTF-8 -*-
-## Copyright 2012 Luc Saffre
-## This file is part of the Lino project.
-## Lino is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
-## (at your option) any later version.
-## Lino is distributed in the hope that it will be useful, 
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
-## GNU General Public License for more details.
-## You should have received a copy of the GNU General Public License
-## along with Lino; if not, see <http://www.gnu.org/licenses/>.
+# Copyright 2012 Luc Saffre
+# License: BSD (see file COPYING for details)
 
 u"""
 
@@ -47,17 +37,17 @@ den man in so einem Fall von einer Shell aus aufrufen könnte.
 from django.conf import settings
 from django.core.management.base import BaseCommand, CommandError
 
-from lino import dd
+from lino import dd, rt
+
 
 class Command(BaseCommand):
     args = '<model> <id>'
     help = 'Validate an existing SSDN request against the xsd files.'
 
     def handle(self, *args, **options):
-        model = dd.resolve_model('cbss.'+args[0])
+        model = dd.resolve_model('cbss.' + args[0])
         pk = int(args[1])
         req = model.objects.get(pk=pk)
         req.validate_request()
         #~ print req.logged_messages
         print req.debug_messages
-      
